﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Numerics;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using OxyPlot;
using OxyPlot.Axes;
using OxyPlot.Series;

namespace WindowsFormsApplication1
{
    public partial class Form1 : Form
    {
        PlotModel _pm = new PlotModel();

        public Form1()
        {
            InitializeComponent();
        }

        void InitializePlotView(PlotModel pm, Rectangle area)
        {
            var axesX = new LinearAxis();
            axesX.Position = AxisPosition.Bottom;
            axesX.AxislineStyle = LineStyle.Solid;
            axesX.PositionAtZeroCrossing = true;

            axesX.Maximum = area.Width;
            // axesX.Minimum = area.X;
            axesX.Minimum = -area.Width;

            axesX.MajorGridlineStyle = LineStyle.Dash;
            axesX.MinorGridlineStyle = LineStyle.Dot;

            var axesY = new LinearAxis();
            axesY.Position = AxisPosition.Left;
            axesY.AxislineStyle = LineStyle.Solid;
            axesY.PositionAtZeroCrossing = true;

            axesY.Maximum = area.Height;
            // axesY.Minimum = area.Y;
            axesY.Minimum = -area.Height;

            axesY.MajorGridlineStyle = LineStyle.Dash;
            axesY.MinorGridlineStyle = LineStyle.Dot;

            pm.Axes.Add(axesX);
            pm.Axes.Add(axesY);

            plotView.Model = pm;
        }

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            Complex[] S = new Complex[]
                {
                    new Complex(2, 2),
                    new Complex(3, 2),
                    new Complex(1.75, 1),
                    new Complex(2, 1),
                    new Complex(2.25, 1),
                    new Complex(2.5, 1),
                    new Complex(2.75, 1),
                    new Complex(3, 1),
                    new Complex(3.25, 1),
                };

            Complex z = new Complex(1, 2);
            // Complex z = new Complex(0, 0);

            // scaling
            var tS1 = from item in S
                      select item * (1.0 / 2);

            // rotation - 180
            var tS2 = from item in S
                      select item * -1;

            // rotation - 90
            z = new Complex(0, 1);
            var tS3 = from item in S
                      select item * z;

            var tS = new List<Complex>();

            tS.AddRange(tS1);
            tS.AddRange(tS2);
            tS.AddRange(tS3);

            var images = FromImage("img01.png");
            tS.AddRange(images);

            ScatterSeries series = PlotsFromComplex(tS);
            Rectangle area = GetAxesArea(series, 3);

            InitializePlotView(_pm, area);

            _pm.Series.Add(series);
        }

        private IEnumerable<Complex> FromImage(string filePath)
        {
            List<Complex> list = new List<Complex>();

            using (Bitmap bitmap = Image.FromFile(filePath) as Bitmap)
            {

                for (int x = 0; x < bitmap.Width; x++)
                {
                    for (int y = 0; y < bitmap.Height; y++)
                    {
                        Color color = bitmap.GetPixel(x, y);
                        if (color.GetBrightness() < 0.5)
                        {
                            Complex z = new Complex(x, -y + bitmap.Height);
                            list.Add(z);
                        }
                    }
                }
            }

            return list;
        }

        private Rectangle GetAxesArea(ScatterSeries series, int margin)
        {
            return GetAxesArea(series, margin, margin);
        }

        private Rectangle GetAxesArea(ScatterSeries series, int xMargin, int yMargin)
        {
            double minX = 0 - xMargin;
            double maxX = 0 + xMargin;

            double minY = 0 - yMargin;
            double maxY = 0 + yMargin;

            foreach (var item in series.Points)
            {
                if (item.X - xMargin < minX)
                {
                    minX = item.X - xMargin;
                }

                if (item.X + xMargin > maxX)
                {
                    maxX = item.X + xMargin;
                }

                if (item.Y - yMargin < minY)
                {
                    minY = item.Y - yMargin;
                }

                if (item.Y + yMargin > maxY)
                {
                    maxY = item.Y + yMargin;
                }
            }

            return new Rectangle((int)minX, (int)minY, (int)maxX, (int)maxY);
        }

        private ScatterSeries PlotsFromComplex(IEnumerable<Complex> complex)
        {
            ScatterSeries series = new ScatterSeries();
            series.MarkerType = MarkerType.Circle;
            series.MarkerSize = 3;
            series.Add(complex);

            return series;
        }
    }
}
